searchData={"items":[{"type":"module","title":"Hermes.Client","doc":"A GenServer implementation of an MCP (Model Context Protocol) client.\n\nThis module handles the client-side implementation of the MCP protocol,\nincluding initialization, request/response handling, and maintaining\nprotocol state.","ref":"Hermes.Client.html"},{"type":"module","title":"Examples - Hermes.Client","doc":"# Start a client process\n    {:ok, client} = Hermes.Client.start_link(\n      name: MyApp.MCPClient,\n      transport: Hermes.Transport.STDIO,\n      client_info: %{\"name\" => \"MyApp\", \"version\" => \"1.0.0\"},\n      capabilities: %{\"resources\" => %{}, \"tools\" => %{}}\n    )\n\n    # List available resources\n    {:ok, resources} = Hermes.Client.list_resources(client)","ref":"Hermes.Client.html#module-examples"},{"type":"module","title":"Notes - Hermes.Client","doc":"The initial client <> server handshake is performed automatically when the client is started.","ref":"Hermes.Client.html#module-notes"},{"type":"function","title":"Hermes.Client.call_tool/4","doc":"Calls a tool on the server.","ref":"Hermes.Client.html#call_tool/4"},{"type":"function","title":"Options - Hermes.Client.call_tool/4","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#call_tool/4-options"},{"type":"function","title":"Hermes.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Client.html#child_spec/1"},{"type":"function","title":"Hermes.Client.close/1","doc":"Closes the client connection and terminates the process.","ref":"Hermes.Client.html#close/1"},{"type":"function","title":"Hermes.Client.get_prompt/4","doc":"Gets a specific prompt from the server.","ref":"Hermes.Client.html#get_prompt/4"},{"type":"function","title":"Options - Hermes.Client.get_prompt/4","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#get_prompt/4-options"},{"type":"function","title":"Hermes.Client.get_schema/1","doc":"","ref":"Hermes.Client.html#get_schema/1"},{"type":"function","title":"Hermes.Client.get_server_capabilities/1","doc":"Gets the server's capabilities as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.html#get_server_capabilities/1"},{"type":"function","title":"Hermes.Client.get_server_info/1","doc":"Gets the server's information as reported during initialization.\n\nReturns `nil` if the client has not been initialized yet.","ref":"Hermes.Client.html#get_server_info/1"},{"type":"function","title":"Hermes.Client.list_prompts/2","doc":"Lists available prompts from the server.","ref":"Hermes.Client.html#list_prompts/2"},{"type":"function","title":"Options - Hermes.Client.list_prompts/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_prompts/2-options"},{"type":"function","title":"Hermes.Client.list_resources/2","doc":"Lists available resources from the server.","ref":"Hermes.Client.html#list_resources/2"},{"type":"function","title":"Options - Hermes.Client.list_resources/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_resources/2-options"},{"type":"function","title":"Hermes.Client.list_tools/2","doc":"Lists available tools from the server.","ref":"Hermes.Client.html#list_tools/2"},{"type":"function","title":"Options - Hermes.Client.list_tools/2","doc":"* `:cursor` - Pagination cursor for continuing a previous request\n  * `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#list_tools/2-options"},{"type":"function","title":"Hermes.Client.merge_capabilities/2","doc":"Merges additional capabilities into the client's capabilities.","ref":"Hermes.Client.html#merge_capabilities/2"},{"type":"type","title":"Hermes.Client.option/0","doc":"","ref":"Hermes.Client.html#t:option/0"},{"type":"function","title":"Hermes.Client.parse_options/1","doc":"","ref":"Hermes.Client.html#parse_options/1"},{"type":"function","title":"Hermes.Client.parse_options!/1","doc":"","ref":"Hermes.Client.html#parse_options!/1"},{"type":"function","title":"Hermes.Client.ping/2","doc":"Sends a ping request to the server to check connection health. Returns `:pong` if successful.","ref":"Hermes.Client.html#ping/2"},{"type":"function","title":"Hermes.Client.read_resource/3","doc":"Reads a specific resource from the server.","ref":"Hermes.Client.html#read_resource/3"},{"type":"function","title":"Options - Hermes.Client.read_resource/3","doc":"* `:timeout` - Request timeout in milliseconds","ref":"Hermes.Client.html#read_resource/3-options"},{"type":"function","title":"Hermes.Client.start_link/1","doc":"Starts a new MCP client process.","ref":"Hermes.Client.html#start_link/1"},{"type":"function","title":"Options - Hermes.Client.start_link/1","doc":"* `:name` - Optional name to register the client process\n  * `:transport` - The transport process or name to use (required)\n  * `:client_info` - Information about the client (required)\n  * `:capabilities` - Client capabilities to advertise\n  * `:protocol_version` - Protocol version to use (defaults to \"2024-11-05\")\n  * `:request_timeout` - Default timeout for requests in milliseconds (default: 30s)","ref":"Hermes.Client.html#start_link/1-options"},{"type":"module","title":"Hermes.Message","doc":"Handles parsing and validation of MCP (Model Context Protocol) messages using the Peri library.\n\nThis module provides functions to parse and validate MCP messages based on the Model Context Protocol schema","ref":"Hermes.Message.html"},{"type":"function","title":"Hermes.Message.decode/1","doc":"Decodes raw data (possibly containing multiple messages) into JSON-RPC messages.\n\nReturns either:\n- `{:ok, messages}` where messages is a list of parsed JSON-RPC messages\n- `{:error, reason}` if parsing fails","ref":"Hermes.Message.html#decode/1"},{"type":"function","title":"Hermes.Message.encode_notification/1","doc":"Encodes a notification message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.Message.html#encode_notification/1"},{"type":"function","title":"Hermes.Message.encode_request/2","doc":"Encodes a request message to a JSON-RPC 2.0 compliant string.\n\nReturns the encoded string with a newline character appended.","ref":"Hermes.Message.html#encode_request/2"},{"type":"function","title":"Hermes.Message.error_schema/1","doc":"","ref":"Hermes.Message.html#error_schema/1"},{"type":"function","title":"Hermes.Message.error_schema!/1","doc":"","ref":"Hermes.Message.html#error_schema!/1"},{"type":"function","title":"Hermes.Message.get_schema/1","doc":"","ref":"Hermes.Message.html#get_schema/1"},{"type":"macro","title":"Hermes.Message.is_error/1","doc":"Determines if a JSON-RPC message is an error.","ref":"Hermes.Message.html#is_error/1"},{"type":"macro","title":"Hermes.Message.is_notification/1","doc":"Determines if a JSON-RPC message is a notification.","ref":"Hermes.Message.html#is_notification/1"},{"type":"macro","title":"Hermes.Message.is_request/1","doc":"Determines if a JSON-RPC message is a request.","ref":"Hermes.Message.html#is_request/1"},{"type":"macro","title":"Hermes.Message.is_response/1","doc":"Determines if a JSON-RPC message is a response.","ref":"Hermes.Message.html#is_response/1"},{"type":"function","title":"Hermes.Message.mcp_message_schema/1","doc":"","ref":"Hermes.Message.html#mcp_message_schema/1"},{"type":"function","title":"Hermes.Message.mcp_message_schema!/1","doc":"","ref":"Hermes.Message.html#mcp_message_schema!/1"},{"type":"function","title":"Hermes.Message.notification_schema/1","doc":"","ref":"Hermes.Message.html#notification_schema/1"},{"type":"function","title":"Hermes.Message.notification_schema!/1","doc":"","ref":"Hermes.Message.html#notification_schema!/1"},{"type":"function","title":"Hermes.Message.request_schema/1","doc":"","ref":"Hermes.Message.html#request_schema/1"},{"type":"function","title":"Hermes.Message.request_schema!/1","doc":"","ref":"Hermes.Message.html#request_schema!/1"},{"type":"function","title":"Hermes.Message.response_schema/1","doc":"","ref":"Hermes.Message.html#response_schema/1"},{"type":"function","title":"Hermes.Message.response_schema!/1","doc":"","ref":"Hermes.Message.html#response_schema!/1"},{"type":"function","title":"Hermes.Message.validate_message/1","doc":"Validates a decoded JSON message to ensure it complies with the MCP schema.","ref":"Hermes.Message.html#validate_message/1"},{"type":"behaviour","title":"Hermes.Transport.Behaviour","doc":"Defines the behavior that all transport implementations must follow.","ref":"Hermes.Transport.Behaviour.html"},{"type":"type","title":"Hermes.Transport.Behaviour.message/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:message/0"},{"type":"type","title":"Hermes.Transport.Behaviour.reason/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:reason/0"},{"type":"callback","title":"Hermes.Transport.Behaviour.send_message/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:send_message/1"},{"type":"callback","title":"Hermes.Transport.Behaviour.send_message/2","doc":"","ref":"Hermes.Transport.Behaviour.html#c:send_message/2"},{"type":"callback","title":"Hermes.Transport.Behaviour.start_link/1","doc":"","ref":"Hermes.Transport.Behaviour.html#c:start_link/1"},{"type":"type","title":"Hermes.Transport.Behaviour.t/0","doc":"","ref":"Hermes.Transport.Behaviour.html#t:t/0"},{"type":"module","title":"Hermes.Transport.STDIO","doc":"A transport implementation that uses standard input/output.\n\nThis module is a `Hermes.Transport.Behaviour` implementation.","ref":"Hermes.Transport.STDIO.html"},{"type":"module","title":"Example - Hermes.Transport.STDIO","doc":"{:ok, stdio} =\n      STDIO.start_link(\n        command: \"mcp\",\n        args: [\"run\", server_path],\n        client: :stdio_test\n      )","ref":"Hermes.Transport.STDIO.html#module-example"},{"type":"function","title":"Hermes.Transport.STDIO.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Hermes.Transport.STDIO.html#child_spec/1"},{"type":"function","title":"Hermes.Transport.STDIO.get_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#get_schema/1"},{"type":"type","title":"Hermes.Transport.STDIO.option/0","doc":"The parameters to start the transport.\n\n- `:command` - The command to execute\n- `:args` - The arguments to pass to the command, as a list of strings\n- `:env` - The environment variables to set for the command, as a map\n- `:cwd` - The working directory for the command, as a string\n- `:client` - The client to send messages to, as a PID or registered name (atom)","ref":"Hermes.Transport.STDIO.html#t:option/0"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema/1"},{"type":"function","title":"Hermes.Transport.STDIO.options_schema!/1","doc":"","ref":"Hermes.Transport.STDIO.html#options_schema!/1"},{"type":"type","title":"Hermes.Transport.STDIO.params_t/0","doc":"","ref":"Hermes.Transport.STDIO.html#t:params_t/0"},{"type":"task","title":"mix stdio.echo","doc":"Mix task to test the STDIO transport implementation.","ref":"Mix.Tasks.Stdio.Echo.html"},{"type":"function","title":"Mix.Tasks.Stdio.Echo.run/1","doc":"","ref":"Mix.Tasks.Stdio.Echo.html#run/1"},{"type":"task","title":"mix stdio.echo.interactive","doc":"Mix task to test the STDIO transport implementation, interactively sending commands.","ref":"Mix.Tasks.Stdio.Echo.Interactive.html"},{"type":"function","title":"Mix.Tasks.Stdio.Echo.Interactive.run/1","doc":"","ref":"Mix.Tasks.Stdio.Echo.Interactive.html#run/1"},{"type":"extras","title":"Hermes MCP","doc":"# Hermes MCP\n\nA high-performance Model Context Protocol (MCP) implementation in Elixir with first-class Phoenix support.","ref":"readme.html"},{"type":"extras","title":"Overview - Hermes MCP","doc":"Hermes MCP provides a unified solution for building both MCP clients and servers in Elixir, leveraging the language's exceptional concurrency model and fault tolerance capabilities. The library offers seamless integration with Phoenix for HTTP/SSE transport while maintaining support for standard stdio communication.","ref":"readme.html#overview"},{"type":"extras","title":"Features - Hermes MCP","doc":"The library implements the full [Model Context Protocol specification](https://spec.modelcontextprotocol.io/specification/2024-11-05/), providing:\n\n- Complete client and server implementations with protocol lifecycle management \n- First-class Phoenix integration for HTTP/SSE transport\n- Robust stdio transport for local process communication\n- Built-in connection supervision and automatic recovery\n- Comprehensive capability negotiation","ref":"readme.html#features"},{"type":"extras","title":"Installation - Hermes MCP","doc":"Add Hermes MCP to your dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:hermes_mcp, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Architecture - Hermes MCP","doc":"The library is structured around several core components:\n\n- Protocol Layer: Handles message framing, request/response correlation, and lifecycle management\n- Transport Layer: Provides pluggable transports for stdio and HTTP/SSE communication\n- Server Components: Implements server-side protocol operations with Phoenix integration\n- Client Components: Manages client-side operations and capability negotiation\n- Supervision Trees: Ensures fault tolerance and automatic recovery\n\nCheck out our technical [RFC](rfc.md) that describe each component more in deep.","ref":"readme.html#architecture"},{"type":"extras","title":"Development Status - Hermes MCP","doc":"Hermes MCP is currently under active development. The initial release will focus on providing stable protocol implementation focused into client interface to existing MCP servers. The API is subject to change as we refine the design based on real-world usage patterns.\n\nWe encourage you to star the repository and watch for updates as we progress toward our first stable release.","ref":"readme.html#development-status"},{"type":"extras","title":"Contributing - Hermes MCP","doc":"We welcome contributions to Hermes MCP! The project is in its early stages, and we're particularly interested in feedback on the design and architecture. Please check our [Issues](https://github.com/cloudwalk/hermes-mcp/issues) page for ways to contribute.","ref":"readme.html#contributing"},{"type":"extras","title":"License - Hermes MCP","doc":"Hermes MCP is released under the MIT License. See [LICENSE](LICENSE) for details.","ref":"readme.html#license"},{"type":"extras","title":"Why Hermes? - Hermes MCP","doc":"The library is named after Hermes, the Greek god of boundaries, communication, and commerce. This namesake reflects the core purpose of the Model Context Protocol: to establish standardized communication between AI applications and external tools. Like Hermes who served as a messenger between gods and mortals, this library facilitates seamless interaction between Large Language Models and various data sources or tools.\n\nFurthermore, Hermes was known for his speed and reliability in delivering messages, which aligns with our implementation's focus on high performance and fault tolerance in the Elixir ecosystem. The name also draws inspiration from the Hermetic tradition of bridging different domains of knowledge, much like how MCP bridges the gap between AI models and external capabilities.","ref":"readme.html#why-hermes"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nAll notable changes to this project are documented in this file.\n\n## [0.1.0](https://github.com/cloudwalk/hermes-mcp) - 2025-02-26","ref":"changelog.html"},{"type":"extras","title":"Added - Changelog","doc":"- Implemented STDIO transport (#1) for MCP communication\n  - Support for bidirectional communication via standard I/O\n  - Automatic process monitoring and recovery\n  - Environment variable handling for cross-platform support\n  - Integration test utilities in Mix tasks\n\n- Created stateful client interface (#6)\n  - Robust GenServer implementation for MCP client\n  - Automatic initialization and protocol handshake\n  - Synchronous-feeling API over asynchronous transport\n  - Support for all MCP operations (ping, resources, prompts, tools)\n  - Proper error handling and logging\n  - Capability negotiation and management\n\n- Developed JSON-RPC message parsing (#5)\n  - Schema-based validation of MCP messages\n  - Support for requests, responses, notifications, and errors\n  - Comprehensive test suite for message handling\n  - Encoding/decoding functions with proper validation\n\n- Established core architecture and client API\n  - MCP protocol implementation following specification\n  - Client struct for maintaining connection state\n  - Request/response correlation with unique IDs\n  - Initial transport abstraction layer","ref":"changelog.html#added"},{"type":"extras","title":"Documentation - Changelog","doc":"- Added detailed RFC document describing the library architecture\n- Enhanced README with project overview and installation instructions","ref":"changelog.html#documentation"},{"type":"extras","title":"RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"# RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","ref":"rfc.html"},{"type":"extras","title":"1. Introduction - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"This document defines the design and architecture of an Elixir library that implements both the client and server aspects of the Model Context Protocol (MCP). The goal is to provide a unified solution that supports robust JSON‑RPC 2.0 communication, leverages Elixir's concurrency and fault tolerance, and integrates seamlessly with Phoenix for server operations. The library will expose a high‑level client API that abstracts transport details and a server API that is compliant with Phoenix conventions for building HTTP endpoints. The design is modular, extensible, and structured to support future enhancements.\n\nFor further context on MCP, see the [Anthropic announcement](https://www.anthropic.com/news/model-context-protocol) and the [Glama Quickstart](https://glama.ai/blog/2024-11-25-model-context-protocol-quickstart).\n\n---","ref":"rfc.html#1-introduction"},{"type":"extras","title":"2. Library Architecture Overview - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"","ref":"rfc.html#2-library-architecture-overview"},{"type":"extras","title":"2.1 General Structure - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library is divided into two main components: the Client and the Server. Both components share core infrastructure for JSON‑RPC handling, configuration, and error management. The Client is designed to provide a high‑level API that abstracts connection management and messaging details, while the Server exposes MCP functionality through Phoenix‑compliant endpoints.\n\nA central configuration module governs the behavior of both components, allowing runtime customization of timeouts, protocol versions, and transport preferences. Middleware hooks are provided at strategic points to support additional processing such as authentication, logging, or transformation.","ref":"rfc.html#2-1-general-structure"},{"type":"extras","title":"2.2 Core Components - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"At the lowest level, a transport abstraction layer decouples the specifics of STDIO, HTTP, or other transports from the core logic. The connection manager (implemented as a GenServer) oversees the client's lifecycle, including the initialization handshake and message correlation. For the server, a dispatcher module routes incoming JSON‑RPC messages (received via a Phoenix controller or Plug) to the appropriate tool handler registered in a tool registry. These components are organized under a comprehensive supervision tree to ensure fault tolerance and automatic recovery.\n\n---","ref":"rfc.html#2-2-core-components"},{"type":"extras","title":"3. Protocol Lifecycle Management - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The Model Context Protocol defines specific lifecycle requirements that both clients and servers must implement. Our Elixir library provides unified lifecycle management that serves both roles through shared infrastructure while maintaining clear separation of concerns.","ref":"rfc.html#3-protocol-lifecycle-management"},{"type":"extras","title":"3.1 Shared Infrastructure - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library implements the protocol lifecycle through a state machine that governs connection establishment, message exchange, and termination. This shared infrastructure leverages Elixir's GenServer and supervision capabilities to provide robust state management and fault tolerance regardless of whether the library is operating in client or server mode.\n\n```mermaid\nstateDiagram-v2\n    [*] --> Initializing\n    Initializing --> Active: Capability Exchange Complete\n    Active --> Recovering: Connection Issue\n    Recovering --> Initializing: Retry Connection\n    Active --> Terminating: Shutdown Requested\n    Recovering --> Terminating: Recovery Failed\n    Terminating --> [*]\n```","ref":"rfc.html#3-1-shared-infrastructure"},{"type":"extras","title":"3.2 Client Lifecycle - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"When operating as a client, the library actively initiates and manages connections to MCP servers. The client connection manager handles the following responsibilities:\n\nThe client begins by establishing a transport connection and sending an initialization request containing its capabilities and protocol version. Upon receiving the server's capability response, it validates compatibility and transitions to the active state. Throughout the session, it maintains correlation between requests and responses while monitoring connection health.\n\nDuring recovery scenarios, the client connection manager attempts to reestablish the transport connection and reinitialize the protocol session. It provides configurable retry policies and transparent recovery when possible.","ref":"rfc.html#3-2-client-lifecycle"},{"type":"extras","title":"3.3 Server Lifecycle - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"As a server, the library acts as a passive participant in lifecycle management, responding to client initiatives while maintaining session isolation. The server session manager creates dedicated processes for each client connection, ensuring proper resource cleanup on termination.\n\nWhen receiving a client's initialization request, the server validates the protocol version compatibility and responds with its supported capabilities. It then maintains session state for the connection, tracking active subscriptions and managing resource access. The server uses Phoenix's connection management facilities when operating over HTTP/SSE transport.","ref":"rfc.html#3-3-server-lifecycle"},{"type":"extras","title":"3.4 Lifecycle Integration - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The following sequence diagram illustrates how the client and server components interact during a typical protocol lifecycle:\n\n```mermaid\nsequenceDiagram\n    participant Client\n    participant Transport\n    participant Server\n    \n    Note over Client,Server: Transport Establishment\n    Client->>Transport: Connect\n    Transport->>Server: Accept Connection\n    \n    Note over Client,Server: Initialization Phase\n    Client->>Server: Initialize Request (capabilities)\n    Server-->>Client: Initialize Response (capabilities)\n    Client->>Server: Initialized Notification\n    \n    Note over Client,Server: Active Session\n    Client->>Server: Protocol Messages\n    Server-->>Client: Protocol Responses\n    \n    Note over Client,Server: Termination\n    Client->>Server: Close Connection\n    Server-->>Client: Acknowledge Close\n```","ref":"rfc.html#3-4-lifecycle-integration"},{"type":"extras","title":"3.5 Error Management - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The library implements comprehensive error handling for both client and server roles. Errors are categorized into:\n\nProtocol Errors: Version mismatches or capability incompatibilities are handled through standardized error responses that maintain protocol compliance.\n\nTransport Errors: Connection failures trigger appropriate recovery mechanisms based on the component's role. Clients attempt reconnection while servers clean up affected sessions.\n\nApplication Errors: Errors during tool execution or resource access are propagated through the protocol's error reporting mechanisms while maintaining session stability.\n\nThis unified approach to lifecycle management ensures protocol compliance while providing robust operation in both client and server roles. The library's supervision strategies and process isolation enable reliable recovery from failure scenarios while maintaining proper resource cleanup.","ref":"rfc.html#3-5-error-management"},{"type":"extras","title":"4. Engineering Considerations - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"","ref":"rfc.html#4-engineering-considerations"},{"type":"extras","title":"4.1 Client Implementation - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The client's core responsibility is to manage the connection and message exchange with an MCP server. A GenServer will serve as the connection manager, maintaining state (such as pending message IDs and associated callbacks) and handling the initialization handshake. The transport layer will be defined by a set of behaviors, which will allow developers to implement additional transport modules (e.g., STDIO or WebSocket) without modifying the core client logic. Error handling is performed through standardized mappings to Elixir's error tuples, and configurable timeouts are implemented to avoid blocking calls.","ref":"rfc.html#4-1-client-implementation"},{"type":"extras","title":"4.2 Server Implementation - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"For the server, Phoenix integration is a primary design goal. The library will provide a Plug/Phoenix module that acts as the HTTP interface for MCP. Incoming JSON‑RPC messages are first parsed and validated, then dispatched to the correct tool handler as registered in the tool registry. The dispatcher module uses asynchronous Tasks to invoke tool handlers, ensuring that long‑running operations do not delay the overall request-response cycle. Session management is achieved by isolating client interactions within dedicated processes, which allows the server to maintain state and context for each connection independently. Additionally, extensive logging and observability hooks are built in to facilitate debugging and performance monitoring.","ref":"rfc.html#4-2-server-implementation"},{"type":"extras","title":"4.3 Shared and Extensible Architecture - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"Both the client and server components share a common configuration module that reads settings from Elixir's configuration files (e.g., config.exs). Middleware hooks are available for both sides, enabling developers to insert custom authentication, logging, or transformation logic into the processing pipeline. The design adheres to a strict supervision tree structure to automatically recover from process failures and to maintain high availability. This shared architecture not only simplifies maintenance but also ensures that future enhancements can be integrated with minimal disruption.","ref":"rfc.html#4-3-shared-and-extensible-architecture"},{"type":"extras","title":"5. Roadmap for Version 0.1.0 - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The roadmap for version 0.1.0 focuses on delivering a robust and well-tested client interface before expanding to full server functionality.","ref":"rfc.html#5-roadmap-for-version-0-1-0"},{"type":"extras","title":"5.1 Phase 1: Design and Requirements (Weeks 1-2) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"During this phase, a detailed design document is created based on the MCP specification. This includes defining the minimal set of features for the client, specifying the public API functions (connect/1, request/2, notify/2), and designing the transport abstraction layer. Prototypes for the connection manager are built and validated against simulated MCP server responses.","ref":"rfc.html#5-1-phase-1-design-and-requirements-weeks-1-2"},{"type":"extras","title":"5.2 Phase 2: Core Client Module Implementation (Weeks 3-5) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"In this phase, the core client module is developed. The connection manager GenServer is implemented, handling the JSON‑RPC handshake and managing state for pending requests. Basic error handling and timeout logic are integrated, and unit tests are written for each component. The transport abstraction is implemented initially for STDIO, with a clear interface for future extensions.","ref":"rfc.html#5-2-phase-2-core-client-module-implementation-weeks-3-5"},{"type":"extras","title":"5.3 Phase 3: Transport Abstraction and High-Level API (Weeks 6-7) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"The focus shifts to refining the transport abstraction layer, ensuring that it supports multiple transport mechanisms seamlessly. The high‑level client API is finalized to provide idiomatic functions that hide low‑level details. Documentation is updated with usage examples and configuration guidelines.","ref":"rfc.html#5-3-phase-3-transport-abstraction-and-high-level-api-weeks-6-7"},{"type":"extras","title":"5.4 Phase 4: Integration Testing and Documentation (Weeks 8-9) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"Comprehensive integration tests are performed by simulating connections to a basic test MCP server. Detailed documentation, including API reference guides and example applications, is produced. Performance metrics, logging outputs, and error scenarios are verified to ensure that the client component meets production standards.","ref":"rfc.html#5-4-phase-4-integration-testing-and-documentation-weeks-8-9"},{"type":"extras","title":"5.5 Phase 5: Beta Release and Feedback (Week 10) - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"A beta version of version 0.1.0 is released to early adopters. Feedback is collected regarding usability, stability, and performance. Based on this feedback, necessary adjustments are made before finalizing the release. This phase sets the stage for subsequent versions that will incorporate full server functionality with Phoenix integration and additional transport support.","ref":"rfc.html#5-5-phase-5-beta-release-and-feedback-week-10"},{"type":"extras","title":"6. Conclusion - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"This RFC has outlined the complete technical design for an Elixir MCP library that provides both client and server capabilities. The public API is designed to be high‑level for the client—abstracting connection and message handling—and fully compliant with Phoenix for the server side. The document details the architecture, internal flows, and the phased roadmap for version 0.1.0, focusing initially on establishing a robust client interface. The modular, extensible design ensures that the library can evolve to support additional features such as dynamic tool registration, middleware enhancements, and alternative transport mechanisms.","ref":"rfc.html#6-conclusion"},{"type":"extras","title":"References - RFC: Elixir MCP Library – Unified Client and Server with Phoenix Integration","doc":"1.\tIntroducing the Model Context Protocol – Anthropic\n2.\tMCP Quickstart – Glama\n3.\tModel Context Protocol – Linkup","ref":"rfc.html#references"},{"type":"extras","title":"LICENSE","doc":"Copyright 2025 zoedsoupe  \n\nPermission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}